#ifndef PORT_H_INCLUDED_
#define PORT_H_INCLUDED_

#include "rose.h"

#include "module.h"
#include "object.h"

namespace risc {

class Port: public Object {

public:

  /**
   * \brief default constructor
   */
  explicit
  Port(SgVariableDefinition *ast_node, Class *parent, Type type = PortType);

  /**
   * \brief Get function for the name of the ast type of the port
   * \return Name as std::string
   */
  virtual std::string get_ast_type_name();

  Class *parent_;

  /**
   * \fn get_mapped_channels
   * \brief This function determines all potential mapped channels to this port
   *
   * CAUTION:
   * If you want to get the specific mapped channel,
   * you have to use the path instance mapper.
   */
  void get_mapped_channels(std::set<risc::Object*> &channels);

  /**
   * \fn get_mapped_channels_types
   * \brief This function determines all potential mapped channels types
   *        to this port
   *
   * CAUTION:
   * If you want to get the specific mapped channel,
   * you have to use the path instance mapper.
   */
  void get_mapped_channel_types(std::set<risc::Class*> &channels);
  
  /*
  * after loading psg, this flag of the loaded port is set to true
  *
  */
  bool bounded;
private:

  /**
   * \brief Disabled copy constructor
   */
  Port(const Port &p);
};

typedef std::vector<Port*> PortVector;
typedef std::vector<Port*>::iterator PortVectorIter;
typedef std::vector<Port*>::const_iterator PortVectorConstIter;

} // end of namespace risc

#endif /* PORT_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
