#ifndef DATA_CONFLICT_AST_ATTRIBUTES_H_INCLUDED_
#define DATA_CONFLICT_AST_ATTRIBUTES_H_INCLUDED_

#include <list>

#include "rose.h"

namespace risc {

namespace sg {

/**
 * \class DataConflictAstAttributes
 * \brief This class should be used for the data conflict analysis.
 *        We can mark a SgVariableRef instance which has reading or writing
 *        access or is unused.
 */
class DataConflictAstAttributes: public AstAttribute {
public:

  // for the new rose release
  virtual AstAttribute::OwnershipPolicy getOwnershipPolicy() const ROSE_OVERRIDE
  {
    return CONTAINER_OWNERSHIP;
  }

  DataConflictAstAttributes();

  DataConflictAstAttributes(bool is_conflict_free);


  bool is_conflict_free_;

  enum Status {UNUSED, READ, WRITE};

  // values:
  //  x >=  0: has a valid instance id
  //  x == -1: global variable
  //  x  < -1: no valid instance id
  //  Ids:
  std::set<int> coloring_read_;
  std::set<int> coloring_write_;
};

} // end namespace sg

} // end namespace risc

#endif /* AST_CONFLICT_ATTRIBUTES_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
