#include "misc.h"

bool
risc::tools::is_data_structure(SgType* type)
{
  switch(type->variant())
  {
    case T_CLASS:
    //case T_STRUCT: INFO: does not exists.
      return true;
      break;
    default:
      return false;
  }
}


bool
risc::tools::is_builtin_type(SgType* type)
{
  // copied the fundamental types from
  // http://www.cplusplus.com/doc/tutorial/variables/

  switch(type->variant())
  {
    // character types (4)
    case T_CHAR:
    case T_WCHAR:

    //TODO
    //The types char16_t and char32_t are not defined

    // integer types signed types (5)
    case T_SIGNED_CHAR:
    case T_SIGNED_SHORT:
    case T_SIGNED_INT:
    case T_SIGNED_LONG:
    case T_SIGNED_LONG_LONG:

    // integer types unsigned types (5)
    case T_UNSIGNED_CHAR:
    case T_UNSIGNED_SHORT:
    case T_UNSIGNED_INT:
    case T_UNSIGNED_LONG:
    case T_UNSIGNED_LONG_LONG:

    // floating point types (3)
    case T_FLOAT:
    case T_DOUBLE:
    case T_LONG_DOUBLE:

    // special cases bool, void, null pointer (3)
    case T_BOOL:
    case T_VOID:


    // Rose specific
    case T_SHORT:
    case T_INT:
    case T_LONG:
    case T_LONG_LONG:

      return true;
      break;

    default:
      return false;
  }
}

bool
risc::tools::class_types_are_identical(SgClassDefinition* cd1,
    SgClassDefinition* cd2)
{
  return cd1->get_declaration()->search_for_symbol_from_symbol_table() ==
    cd2->get_declaration()->search_for_symbol_from_symbol_table();
}

void
risc::tools::print_indented(std::string s, unsigned indent)
{
  for(unsigned i=0; i<indent; i++) {
    std::cout << "  " << std::endl;
  }
  std::cout << s << std::endl;

}

bool
risc::tools::is_func_name(const SgFunctionDeclaration* function,
    const std::string& name)
{
  return function && function->get_name().getString() == name;
}

SgClassDefinition*
risc::tools::class_definition_of_variable(SgVariableDeclaration* var_decl)
{

/*** Tim's version, which filters out only pointers ***********************

  SgPointerType *ptr = isSgPointerType(var_decl->get_definition()->get_type());
  SgClassType* type = NULL;

  if(ptr) {

    type = isSgClassType(ptr->get_base_type());
  } else {

    type = isSgClassType(var_decl->get_definition()->get_type());
  }

  SgClassDeclaration* decl_stmt = isSgClassDeclaration(type->get_declaration());
  decl_stmt = isSgClassDeclaration(decl_stmt->get_definingDeclaration());
  SgClassDefinition* class_def = decl_stmt->get_definition();
  return class_def;

 **************************************************************************/

 // Rainer's version, which strips off all upper type layers (bug fix 06/26/18)

  SgClassType *type = isSgClassType(var_decl->get_definition()->get_type()
                                      ->stripType());
  if (!type) { // not a class type, can't provide a class definition!
    return(NULL);
  }

  SgClassDeclaration* decl_stmt = isSgClassDeclaration(type->get_declaration());
  decl_stmt = isSgClassDeclaration(decl_stmt->get_definingDeclaration());
  SgClassDefinition* class_def = decl_stmt->get_definition();
  return class_def;
}

bool
risc::tools::is_builtin_function(SgFunctionDeclaration *func_decl)
{
  std::string filename = func_decl->get_file_info()->get_filename();
  std::string func_name = func_decl->get_name().getString();
  std::string stripped_file_name =
    Rose::StringUtility::stripPathFromFileName(filename);

  if(func_decl->get_file_info()->isCompilerGenerated())
    return false;

  if(func_decl->get_file_info()->isFrontendSpecific())
    return false;

  if( stripped_file_name == "assert.h"  ||
      stripped_file_name == "complex.h" ||
      stripped_file_name == "ctype.h"   ||
      stripped_file_name == "errno.h"   ||
      stripped_file_name == "float.h"   ||

      stripped_file_name == "limits.h"  ||
      stripped_file_name == "locale.h"  ||
      stripped_file_name == "math.h"    ||
      stripped_file_name == "setjmp.h"  ||
      stripped_file_name == "signal.h"  ||

      stripped_file_name == "stdarg.h"  ||
      stripped_file_name == "stddef.h"  ||

      stripped_file_name == "stdio.h"   ||
      stripped_file_name == "stdlib.h"  ||
      stripped_file_name == "string.h"  ||
      stripped_file_name == "time.h"    ||

      stripped_file_name == "libio.h"   ||
      stripped_file_name == "select.h"  ||
      stripped_file_name == "mathcalls.h")
    return true;

  return false;
}

bool
risc::tools::is_located_in_rose_header(SgNode *node)
{

  std::string filename = node->get_file_info()->get_filename();
  std::string stripped_file_name =
    Rose::StringUtility::stripPathFromFileName(filename);

  if( stripped_file_name == "gthr-default.h" ||
      stripped_file_name == "new" ||
      stripped_file_name == "stl_construct.h" ||
      stripped_file_name == "stl_uninitialized.h" ||
      stripped_file_name == "vector.tcc" ||
      stripped_file_name == "stl_vector.h" ||
      stripped_file_name == "stl_uninitialized.h" ||
      stripped_file_name == "stl_uninitialized.h" ||
      stripped_file_name == "rose_edg_required_macros_and_functions.h") {

    return true;
  }
  return false;
}

bool
risc::tools::all_cfg_branches_end_in_break(SgNode *node, int &break_counter)
{
  if(node == NULL) {
    return false;
  }

  if(isSgBreakStmt(node)) {
    break_counter++;
    return true;
  }

  if(isSgBasicBlock(node)) {
    SgBasicBlock *bb = isSgBasicBlock(node);
    SgStatementPtrList &list = bb->get_statements();

    for(std::vector<SgStatement*>::iterator
        iter  = list.begin();
        iter != list.end();
        iter++) {

      if(all_cfg_branches_end_in_break(*iter, break_counter)) {
        return true;
      }
    }
    return false;
  }

  if(isSgIfStmt(node)) {
    SgIfStmt *if_stmt = isSgIfStmt(node);

    SgStatement *true_body = if_stmt->get_true_body();
    SgStatement *false_body = if_stmt->get_false_body();

    if(true_body == NULL || false_body == NULL)
      return false;

    bool true_body_result
      = all_cfg_branches_end_in_break(true_body, break_counter);
    bool false_body_result
      = all_cfg_branches_end_in_break(false_body, break_counter);

    return true_body_result && false_body_result;
  }

  if(isSgSwitchStatement(node)) {
    return false;
  }

  if(isSgWhileStmt(node)) {
    return false;
  }

  if(isSgForStatement(node)) {
    return false;
  }

  if(isSgDoWhileStmt(node)) {
    return false;
  }

  return false;
}


bool
risc::tools::all_cfg_branches_end_in_continue(SgNode *node,
    int &continue_counter)
{
  if(isSgContinueStmt(node)) {
    continue_counter++;
    return true;
  }

  if(isSgBasicBlock(node)) {
    SgBasicBlock *bb = isSgBasicBlock(node);
    SgStatementPtrList &list = bb->get_statements();

    for(std::vector<SgStatement*>::iterator
        iter  = list.begin();
        iter != list.end();
        iter++) {

      if(all_cfg_branches_end_in_continue(*iter, continue_counter)) {
        return true;
      }
    }
    return false;
  }

  if(isSgIfStmt(node)) {
    SgIfStmt *if_stmt = isSgIfStmt(node);

    SgStatement *true_body = if_stmt->get_true_body();
    SgStatement *false_body = if_stmt->get_false_body();

    if(true_body == NULL || false_body == NULL)
      return false;

    bool true_body_result =
      all_cfg_branches_end_in_continue(true_body, continue_counter);
    bool false_body_result =
      all_cfg_branches_end_in_continue(false_body, continue_counter);

    return true_body_result && false_body_result;
  }

  if(isSgSwitchStatement(node)) {
    return false;
  }

  if(isSgWhileStmt(node)) {
    return false;
  }

  if(isSgForStatement(node)) {
    return false;
  }

  if(isSgDoWhileStmt(node)) {
    return false;
  }

  return false;
}


SgStatement*
risc::tools::get_previous_stmt_in_basic_block(SgNode *node)
{
  SgScopeStatement *scope = SageInterface::getScope(node);
  SgStatement *result = NULL;

  SgBasicBlock *bb = isSgBasicBlock(scope);

  if(bb == NULL)
    return NULL;

  SgStatementPtrList &list = bb->get_statements();

  for(std::vector<SgStatement*>::iterator
      iter  = list.begin();
      iter != list.end();
      iter++) {

    if(*iter == node) {
      break;

    } else {
      result = *iter;

    }
  }

  return result;
}

bool
risc::tools::is_conditional_break(
    SgScopeStatement const * const loop_stmt,
    SgBreakStmt const * const break_stmt)
{
  const SgNode* iterator = break_stmt;

  while(iterator != NULL) {

    if(isSgIfStmt(iterator)) {
      return true;
    }

    if(iterator == loop_stmt) {
      return false;
    }

    iterator = iterator->get_parent();
  }

  return false;
}

SgScopeStatement*
risc::tools::find_break_surrounding_scope(SgBreakStmt *break_stmt)
{
  SgNode* iterator = break_stmt;

  while(iterator != NULL) {

    if(isSgDoWhileStmt(iterator)) {
      return isSgDoWhileStmt(iterator);
    }
    if(isSgForStatement(iterator)) {
      return isSgForStatement(iterator);
    }
    if(isSgWhileStmt(iterator)) {
      return isSgWhileStmt(iterator);
    }
    if(isSgSwitchStatement(iterator)) {
      return isSgSwitchStatement(iterator);
    }

    iterator = iterator->get_parent();
  }

  return NULL;
}

bool
risc::tools::break_belongs_to_loop(
    SgScopeStatement *loop_stmt,
    SgBreakStmt const * const break_stmt)
{
  std::vector<SgBreakStmt*> break_stmts
    = SageInterface::findBreakStmts(SageInterface::getLoopBody(loop_stmt));

  if(std::find(break_stmts.begin(), break_stmts.end(), break_stmt) ==
      break_stmts.end()) {

    return false;
  } else {

    return true;
  }
}

SgMemberFunctionDeclaration*
risc::tools::get_constructor(SgClassDefinition *class_def)
{
  std::vector<SgNode*> func_defs
    = NodeQuery::querySubTree(class_def, V_SgFunctionDefinition);

  for(std::vector<SgNode*>::iterator
      func_def_iter  = func_defs.begin();
      func_def_iter != func_defs.end();
      func_def_iter++) {

    SgFunctionDefinition *func_def = isSgFunctionDefinition(*func_def_iter);
    SgFunctionDeclaration *func_decl = func_def->get_declaration();

    if(func_decl->get_specialFunctionModifier().isConstructor()) {

      return isSgMemberFunctionDeclaration(func_decl);

    }
  }
  return NULL;
}

void
risc::tools::find_and_replace_all(
    std::string &input,
    const std::string &from,
    const std::string &to)
{
  // This function is from stack overflow
  size_t start_pos = 0;
  while((start_pos = input.find(from, start_pos)) != std::string::npos) {
    input.replace(start_pos, from.length(), to);

    // Handles case where 'to' is a substring of 'from'
    start_pos += to.length();
  }
}

std::string risc::tools::get_mangled_name(SgClassDefinition *class_def)
{
  std::string mangled_name
    = class_def->get_declaration()->get_type()->get_mangled();
  tools::find_and_replace_all(mangled_name, "__scope__", "::");
  return mangled_name;
}

SgVariableSymbol*
risc::tools::find_global_variable(std::string name, SgProject *project)
{
  for(int file_id = 0; file_id < project->numberOfFiles(); file_id++) {

    SgSourceFile *file
      = isSgSourceFile(&project->get_file(file_id));

    for(SgDeclarationStatementPtrList::iterator
        iter  = file->get_globalScope()->get_declarations().begin();
        iter != file->get_globalScope()->get_declarations().end();
        iter++) {

      SgVariableDeclaration *var_decl = isSgVariableDeclaration(*iter);

      if(var_decl) {
        std::string temp_name = SageInterface::get_name(var_decl);
        temp_name.erase(0, 22);
        if(temp_name == name) {

          return isSgVariableSymbol(var_decl->get_decl_item(0)
                     ->search_for_symbol_from_symbol_table());
        }
      }
    }
  }
}


bool risc::tools::is_known_function(SgFunctionDeclaration *func_decl)
{
  if(func_decl == NULL) {
    return false;
  }

  if(SageInterface::get_name(func_decl) == "printf" ||
    SageInterface::get_name(func_decl) == "free"    ||
    SageInterface::get_name(func_decl) == "putchar" ||
    SageInterface::get_name(func_decl) == "rand"    ||
    SageInterface::get_name(func_decl) == "memset"  ||
    SageInterface::get_name(func_decl) == "scanf" )
    return true;

  return false;
}

bool risc::tools::is_in_std_namespace(SgDeclarationStatement *statement)
{
  if(statement == NULL) {
    return false;
  }

  do {

    SgNamespaceDefinitionStatement *namespace_def
      = SageInterface::enclosingNamespaceScope(statement);

    if(namespace_def == NULL) {
      return false;
    }

    // I am using here a string comparison because there are multiple
    // namespaces with std. So, I don't want to iterate over all of them
    if(SageInterface::get_name(namespace_def) == "std") {
      return true;
    }

    statement = namespace_def->get_namespaceDeclaration();

  } while(statement);

  return false;
}

bool risc::tools::is_in_gnu_cxx_namespace(SgDeclarationStatement *statement)
{
  if(statement == NULL) {
    return false;
  }

  do {

    SgNamespaceDefinitionStatement *namespace_def
      = SageInterface::enclosingNamespaceScope(statement);

    if(namespace_def == NULL) {
      return false;
    }

    // I am using here a string comparison because there are multiple
    // namespaces with std. So, I don't want to iterate over all of them
    if(SageInterface::get_name(namespace_def) == "__gnu_cxx") {
      return true;
    }

    statement = namespace_def->get_namespaceDeclaration();

  } while(statement);

  return false;
}

bool risc::tools::is_in_systemc_namespace(SgDeclarationStatement *statement)
{
  if(statement == NULL) {
    return false;
  }

  do {

    SgNamespaceDefinitionStatement *namespace_def
      = SageInterface::enclosingNamespaceScope(statement);

    if(namespace_def == NULL) {
      return false;
    }

    // I am using here a string comparison because there are multiple
    // namespaces with sc_core/sc_det. So, I don't want to iterate over all of
    // them
    if(SageInterface::get_name(namespace_def) == "sc_core" ||
        SageInterface::get_name(namespace_def) == "sc_dt") {
      return true;
    }

    statement = namespace_def->get_namespaceDeclaration();

  } while(statement);

  return false;
}

SgVariableSymbol* risc::tools::get_most_outer_var(SgExpression *expr)
{
  SgBinaryOp *binary = isSgBinaryOp(expr);
  if(binary) {
    SgThisExp *this_exp = isSgThisExp(binary->get_lhs_operand());
    if(this_exp) {
      return isSgVarRefExp(binary->get_rhs_operand())->get_symbol();
    } else {
      return get_most_outer_var(binary->get_lhs_operand());
    }
  }

  SgUnaryOp *unary = isSgUnaryOp(expr);
  if(unary) {
    return get_most_outer_var(unary->get_operand());
  }

  if(expr) {
    SgVarRefExp *temp = isSgVarRefExp(expr);

    if(temp) {
      return temp->get_symbol();
    } else {
      return NULL;
    }
  } else {
    return NULL;
  }
}

bool risc::tools::has_no_variable_expressions(SgExpression *expr)
{
  std::vector<SgNode*> var_refs
    = NodeQuery::querySubTree(expr, V_SgVarRefExp);

  if(var_refs.empty()) {
    return true;
  } else {
    return false;
  }
}

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
