#!/usr/bin/env python
#
# PerfEval.py: hack to validate RISC simulation performance results from logs
#
# author: Rainer Doemer
#
# modifications:
# 08/20/18 RD	added missing entries for recently added test cases
# 07/11/18 RD	added or_event[123], event_and_list[23], event_or_list[23],
#		event_lists, data_conflict, sensitive[12]
# 06/12/18 RD	added event_or_list, event_and_list, prod_cons_signal,
#		and initial trigger bug20180523
# 06/08/18 RD	added bug20180418 regression test
# 04/19/18 RD	adjusted expected performance for the following test cases:
#		particle_channel1wr, particle_channel2r, particle_channel5r,
#		particle_channel2wr, particle_channel5wr, one_notify_one_wait2,
#		one_notify_one_wait3; with PCP and reference analysis,
#		more parallelism is expected now in RISC v0.4.1
# 04/17/18 RD	added bug20180302 and bug20180309 regression tests
# 04/17/18 RD	added particle_channel2_whiteboard regression test case
# 04/17/18 RD	added player_references regression test case
# 04/13/18 RD	added player_unused_modules regression test case
# 04/12/18 RD	added player_port_binding regression test case
# 08/02/17 RD	added particle_channel5* regression test cases
# 07/20/17 RD	added mandelbrot*_crunch models from RISCv030 (updated in v040)
# 07/14/17 RD	expectations adjusted for RISC 0.4.0 on host 'delta'
# 07/12/17 RD	added the missing bug20170331 and bug20170331b entries,
#		as well as place holder entries for particle_channel4*
# 05/02/17 RD	lowered ClearlyGreater percentage to 8% for
#		player_duplicated_channels*
# 05/02/17 RD	added checking of and_event[123] examples
# 04/28/17 RD	added/updated player* examples
# 04/25/17 RD	initial version

# imports
import sys
import os
import time

# classes:
class Perf:
    "performance values of an example"
    def __init__(self,h,u,s,e,c,x):
	self.h = h
	self.u = u
	self.s = s
	self.e = e
	self.c = c
	self.x = x

# checks to be performed
Checks = [
    ("cyclic_notify",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("green_example",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("immediate_wake_up",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("one_notify_one_wait",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("one_notify_one_wait2",
	("seq.c<=100","ooo.c<=200","syn.c<=110","npd.c<=110",
                      "ooo.c>=100",
	 "seq.e<=5.0","ooo.e<=4.0","syn.e<=5.0","npd.e<=5.0")),
    ("one_notify_one_wait3",
	("seq.c<=100","ooo.c<=200","syn.c<=110","npd.c<=110",
                      "ooo.c>=100",
	 "seq.e<=5.0","ooo.e<=4.0","syn.e<=5.0","npd.e<=5.0")),
    ("one_channel",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("particle_channel1",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("particle_channel2",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("particle_channel3",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
#   ("particle_channel4",
#	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
#	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("particle_channel5",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("particle_channel1w",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("particle_channel2w",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("particle_channel3w",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
#   ("particle_channel4w",
#	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
#	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("particle_channel5w",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("particle_channel1r",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=3.0","ooo.e<=3.0","syn.e<=3.0","npd.e<=3.0")),
    ("particle_channel2r",
	("seq.c<=100","ooo.c<=200","syn.c<=200","npd.c<=200",
	              "ooo.c>=100","syn.c>=100","npd.c>=100",
	 "seq.e<=5.0","ooo.e<=3.0","syn.e<=3.0","npd.e<=3.0")),
    ("particle_channel3r",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=3.0","ooo.e<=3.0","syn.e<=3.0","npd.e<=3.0")),
#   ("particle_channel4r",
#	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
#	 "seq.e<=3.0","ooo.e<=3.0","syn.e<=3.0","npd.e<=3.0")),
    ("particle_channel5r",
	("seq.c<=100","ooo.c<=150","syn.c<=150","npd.c<=150",
	              "ooo.c>=100","syn.c>=100","npd.c>=100",
	 "seq.e<=3.0","ooo.e<=3.0","syn.e<=3.0","npd.e<=3.0")),
    ("particle_channel1wr",
	("seq.c<=100","ooo.c<=200","syn.c<=200","npd.c<=200",
	              "ooo.c>=100","syn.c>=100","npd.c>=100",
	 "seq.e<=3.0","ooo.e<=2.0","syn.e<=2.0","npd.e<=2.0")),
    ("particle_channel2wr",
	("seq.c<=100","ooo.c<=300","syn.c<=300","npd.c<=300",
	              "ooo.c>=200","syn.c>=200","npd.c>=200",
	 "seq.e<=5.0","ooo.e<=3.0","syn.e<=3.0","npd.e<=3.0")),
    ("particle_channel3wr",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=3.0","ooo.e<=3.0","syn.e<=3.0","npd.e<=3.0")),
#   ("particle_channel4wr",
#	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
#	 "seq.e<=3.0","ooo.e<=3.0","syn.e<=3.0","npd.e<=3.0")),
    ("particle_channel5wr",
	("seq.c<=100","ooo.c<=150","syn.c<=150","npd.c<=150",
	              "ooo.c>=100","syn.c>=100","npd.c>=100",
	 "seq.e<=3.0","ooo.e<=3.0","syn.e<=3.0","npd.e<=3.0")),
    ("particle_channel2_whiteboard",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("synchronous_conflict",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("parallel_threads1",
	("seq.c<=100","ooo.c<=200","syn.c<=110","npd.c<=200",
	              "ooo.c>=100",             "npd.c>=100",
	 "seq.e<=2.0","ooo.e<=1.0","syn.e<=2.0","npd.e<=1.0")),
    ("parallel_threads2",
	("seq.c<=100","ooo.c<=400","syn.c<=200","npd.c<=400",
	              "ooo.c>=250","syn.c>=100","npd.c>=250",
	 "seq.e<=3.0","ooo.e<=1.0","syn.e<=1.5","npd.e<=1.0")),
    ("parallel_threads3",
	("seq.c<=100","ooo.c<=800","syn.c<=200","npd.c<=800",
	              "ooo.c>=300","syn.c>=100","npd.c>=300",
	 "seq.e<=3.0","ooo.e<=1.0","syn.e<=1.5","npd.e<=1.0")),
    ("parallel_threads_counting",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("player",
	("seq.c<=100","ooo.c<=300","syn.c<=200","npd.c<=300",
	              "ooo.c>=200","syn.c>=100","npd.c>=200",
	 "AboutEqual(ooo.c,npd.c)",
	 "ClearlyGreater(ooo.c,syn.c)",
	 "ClearlyGreater(syn.c,seq.c)",
	 "seq.e<=3.0","ooo.e<=1.5","syn.e<=2.0","npd.e<=1.5")),
    ("player_duplicated_channels",
	("seq.c<=100","ooo.c<=300","syn.c<=200","npd.c<=300",
	              "ooo.c>=200","syn.c>=100","npd.c>=200",
	 "AboutEqual(ooo.c,npd.c)",
	 "ClearlyGreater(ooo.c,syn.c)",
	 "ClearlyGreater(syn.c,seq.c)",
	 "seq.e<=3.0","ooo.e<=1.5","syn.e<=2.0","npd.e<=1.5")),
    ("playerL",
	("seq.c<=100","ooo.c<=300","syn.c<=200","npd.c<=300",
	              "ooo.c>=200","syn.c>=100","npd.c>=200",
	 "AboutEqual(ooo.c,npd.c)",
	 "ClearlyGreater(ooo.c,syn.c)",
	 "ClearlyGreater(syn.c,seq.c)",
	 "seq.e<=3.0","ooo.e<=1.5","syn.e<=2.0","npd.e<=1.5")),
    ("player_duplicated_channelsL",
	("seq.c<=100","ooo.c<=300","syn.c<=200","npd.c<=300",
	              "ooo.c>=200","syn.c>=100","npd.c>=200",
	 "AboutEqual(ooo.c,npd.c)",
	 "ClearlyGreater(ooo.c,syn.c)",
	 "ClearlyGreater(syn.c,seq.c)",
	 "seq.e<=3.0","ooo.e<=1.5","syn.e<=2.0","npd.e<=1.5")),
    ("player_port_binding",
	("seq.c<=100","ooo.c<=300","syn.c<=200","npd.c<=300",
	              "ooo.c>=200","syn.c>=100","npd.c>=200",
	 "AboutEqual(ooo.c,npd.c)",
	 "ClearlyGreater(ooo.c,syn.c)",
	 "ClearlyGreater(syn.c,seq.c)",
	 "seq.e<=3.0","ooo.e<=1.5","syn.e<=2.0","npd.e<=1.5")),
    ("player_unused_modules",
	("seq.c<=100","ooo.c<=300","syn.c<=200","npd.c<=300",
	              "ooo.c>=200","syn.c>=100","npd.c>=200",
	 "AboutEqual(ooo.c,npd.c)",
	 "ClearlyGreater(ooo.c,syn.c)",
	 "ClearlyGreater(syn.c,seq.c)",
	 "seq.e<=3.0","ooo.e<=1.5","syn.e<=2.0","npd.e<=1.5")),
    ("player_references",
	("seq.c<=100","ooo.c<=300","syn.c<=200","npd.c<=300",
	              "ooo.c>=200","syn.c>=100","npd.c>=200",
	 "AboutEqual(ooo.c,npd.c)",
	 "ClearlyGreater(ooo.c,syn.c)",
	 "ClearlyGreater(syn.c,seq.c)",
	 "seq.e<=3.0","ooo.e<=1.5","syn.e<=2.0","npd.e<=1.5")),
    ("mandelbrot_crunch",
	("seq.c<=100","ooo.c<=800","syn.c<=800","npd.c<=800",
	              "ooo.c>=500","syn.c>=500","npd.c>=500",
	 "AboutEqual(syn.c,npd.c)", "AboutEqual(syn.c,ooo.c)",
	 "seq.e>=70.","ooo.e>=10.","syn.e>=10.","npd.e>=10.",
	 "seq.e<=80.","ooo.e<=20.","syn.e<=20.","npd.e<=20.")),
    ("mandelbrot_fifo_crunch",
	("seq.c<=100","ooo.c<=800","syn.c<=800","npd.c<=800",
	              "ooo.c>=500","syn.c>=500","npd.c>=500",
	 "AboutEqual(syn.c,npd.c)", "AboutEqual(syn.c,ooo.c)",
	 "seq.e>=70.","ooo.e>=10.","syn.e>=10.","npd.e>=10.",
	 "seq.e<=80.","ooo.e<=20.","syn.e<=20.","npd.e<=20.")),
    ("bug20160629",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("bug20170331",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("bug20170331b",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("bug20180302",
	("seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("bug20180309",
	("seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("bug20180418",
	("seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("bug20180523",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("bug20180717a",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("bug20180717b",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
#   ("bug20180719",
#	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
#	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
#   ("bug20180723a",
#	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
#	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
#   ("bug20180723b",
#	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
#	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
#   ("bug20180723c",
#	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
#	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
#   ("bug20180723d",
#	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
#	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("bug20180724",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("prod_cons_signal",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("and_event1",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("and_event2",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("and_event3",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("or_event1",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("or_event2",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("or_event3",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("event_or_list",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("event_or_list2",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("event_or_list3",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("event_and_list",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("event_and_list2",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("event_and_list3",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("data_conflict",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("event_lists",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("sensitive1",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("sensitive2",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("sensitive3",
	("seq.c<=100","ooo.c<=110","syn.c<=110","npd.c<=110",
	 "seq.e<=1.0","ooo.e<=1.0","syn.e<=1.0","npd.e<=1.0")),
    ("sc_start",
	("seq.c<=100","ooo.c<=300","syn.c<=200","npd.c<=300",
	              "ooo.c>=200","syn.c>=100","npd.c>=200",
	 "AboutEqual(ooo.c,npd.c)",
	 "ClearlyGreater(ooo.c,syn.c)",
	 "ClearlyGreater(syn.c,seq.c)",
	 "seq.e<=25.0","ooo.e<=11.0","syn.e<=17.0","npd.e<=11.0")),
    ("sc_start2",
	("seq.c<=100","ooo.c<=300","syn.c<=200","npd.c<=300",
	              "ooo.c>=200","syn.c>=100","npd.c>=200",
	 "AboutEqual(ooo.c,npd.c)",
	 "ClearlyGreater(ooo.c,syn.c)",
	 "ClearlyGreater(syn.c,seq.c)",
	 "seq.e<=25.0","ooo.e<=11.0","syn.e<=17.0","npd.e<=11.0"))]

# functions
def PrintLogLine(d,e):
    print("%s: %0.2f user %0.2f system %0.2f elapsed %d%% CPU %d exit: %s" %
	(d.h, d.u, d.s, d.e, d.c, d.x, e))

def AboutEqual(a,b,Pvar=0.10):
    return (a >= b*(1.0-Pvar)) and (a <= b*(1.0+Pvar))

def ClearlyGreater(a,b,Pvar=0.10):
    return (a > b*(1.0+Pvar))

#
# Action starts here!
#

# parse command line arguments

Cmd = os.path.basename(sys.argv[0])
if len(sys.argv) < 2:
    sys.stderr.write("%s: bad arguments!\n" % Cmd)
    sys.stderr.write("Usage: %s <log file>\n" % Cmd)
    sys.exit(10)

LogName = sys.argv[1]
sys.stderr.write("%s: Reading log file \"%s\"...\n" % (Cmd,LogName))

# read the log file

try:
    File = open(LogName, "r")
except IOError, Msg:
    sys.stderr.write("%s: Cannot open file %s for reading: %s" % (Cmd,LogName,Msg))
    sys.exit(10)
Lines = File.readlines();
File.close()
sys.stderr.write("%s: %d lines read.\n" % (Cmd,len(Lines)))

# parse the contents

Examples = [ ]
Dict = { }
for l in Lines:
    (Host,u,Usr,s,Sys,e,Ela,c,Cpu,x,Exi,CmdLine) = [t(s) for t,s in zip((str,float,str,float,str,float,str,str,str,int,str,str),l.split())]
#   print Host,u,s,e,c,x,CmdLine
    Host = Host[:-1]	# get rid of ':' in host name
    c = int(c[:-1])	# get rid of '%' in CPU utilization
    if CmdLine.startswith("./"):
	CmdLine = CmdLine[2:]
#   print Host,u,s,e,c,x,CmdLine
    if CmdLine in Examples:
	sys.stderr.write("%s: Multiple results for '%s'\n" % (Cmd,CmdLine))
    else:
	Examples.append(CmdLine)
	Dict[CmdLine] = (Host,u,s,e,c,x)
sys.stderr.write("%s: %d result records.\n" % (Cmd,len(Dict)))

# execute the checks

NumChecks = 0
NumTests = 0
NumWarnings = 0
for check in Checks:
    (Name,Tests) = check
    Example = Name+"_seq"
    if Example in Examples:
	Data = Dict[Example]
	seq = Perf(Data[0],Data[1],Data[2],Data[3],Data[4],Data[5])
    else:
	sys.stderr.write("%s: Skipping '%s', no data available!\n"
				% (Cmd,Example))
	continue
    Example = Name+"_ooo"
    if Example in Examples:
	Data = Dict[Example]
	ooo = Perf(Data[0],Data[1],Data[2],Data[3],Data[4],Data[5])
    else:
	sys.stderr.write("%s: Skipping '%s', no data available!\n"
				% (Cmd,Example))
	continue
    Example = Name+"_syn"
    if Example in Examples:
	Data = Dict[Example]
	syn = Perf(Data[0],Data[1],Data[2],Data[3],Data[4],Data[5])
    else:
	sys.stderr.write("%s: Skipping '%s', no data available!\n"
				% (Cmd,Example))
	continue
    Example = Name+"_npd"
    if Example in Examples:
	Data = Dict[Example]
	npd = Perf(Data[0],Data[1],Data[2],Data[3],Data[4],Data[5])
    else:
	sys.stderr.write("%s: Skipping '%s', no data available!\n"
				% (Cmd,Example))
	continue
#   print seq, ooo, syn, npd
    NumChecks += 1
    for Test in Tests:
	NumTests += 1
	if not eval(Test):
	    NumWarnings += 1
	    print "\nWARNING: for %s, test '%s' failed!" % (Name,Test)
	    PrintLogLine(seq, Name+"_seq")
	    PrintLogLine(ooo, Name+"_ooo")
	    PrintLogLine(syn, Name+"_syn")
	    PrintLogLine(npd, Name+"_npd")

# result

if NumWarnings:
    print "\nWARNING: %d tests failed!" % NumWarnings
sys.stderr.write("%s: %d examples checked, a total of %d tests performed.\n"
		% (Cmd, NumChecks, NumTests))
sys.stderr.write("%s: Done.\n" % Cmd)

sys.exit(NumWarnings)

# EOF
