// bug20170331.cc
// 05/26/17 RD        after a lot of analysis, debugging, and code fixing
//                    in both simulator and compiler, I'm adding this test
//                    case into the regression test suite as original trigger
//                    of the question whether or not channel variables need
//                    to be considered by the conflict analysis in the compiler
// 03/31/17 ZC+RD+TS  problematic channel that triggered the regression tests
//                    (added based on email ZC to TS after discussion with RD)

#include <stdio.h>
#include "systemc.h"

#define ITERATIONS 10000

class i_long_long_sender: virtual public sc_interface
{
  public:
    virtual void send(long long d) = 0;
};

class i_long_long_receiver: virtual public sc_interface
{
  public:
    virtual long long receive(void) = 0;
};

class c_bad: // RD: was "c_long_long_queue", but this is NOT a queue!
        public sc_channel,
        public i_long_long_sender, public i_long_long_receiver
{
  public:
    c_bad(sc_module_name name):
                sc_channel(name)
    {
      sent=0;
      res=1;
    }

    ~c_bad(void)
    {
    }

    long long receive(void)
    {
        if(sent==0) wait(s); // RD: 'if' should be 'while'
        sent=0;
        res=1;
        r.notify(SC_ZERO_TIME); // RD: maybe intended as immediate notification
        return val;
    }

    void send(long long d)
    {
        if(res==0) wait(r); // RD: 'if' should be 'while'
        res=0;
        val=d;
        sent=1;
        s.notify(SC_ZERO_TIME); // RD: maybe intended as immediate notification
    }

  private:
    sc_event  s,r;
    int       sent,res;
    long long val;
};

SC_MODULE(S)
{
  sc_port<i_long_long_sender> p1;
  int n;

  void main(void)
  {
    for(int i=0; i<ITERATIONS; i++)
    {
	p1->send(i);
	n++;
    }
    assert(n == ITERATIONS);
  }

  SC_CTOR(S)
  {
    n = 0;
    SC_THREAD(main);
  }
};

SC_MODULE(R)
{
  sc_port<i_long_long_receiver> p1;
  int n;

  void main(void)
  {
    long long d;

    for(int i=0; i<ITERATIONS; i++)
    {
	d = p1->receive();
        assert(d==i);
	n++;
        wait(20, SC_NS); // RD: OOO scheduling must not be possible!
    }
    assert(n == ITERATIONS);
  }

  SC_CTOR(R)
  {
    n = 0;
    SC_THREAD(main);
  }
};

SC_MODULE(Top)
{
  c_bad c;
  S s;
  R r;

  void before_end_of_elaboration()
  {
    s.p1.bind(c);
    r.p1.bind(c);
  }

  SC_CTOR(Top)
  : c("c")
  , r("r")
  , s("s")
  { }
};

Top top("top");

int sc_main(int argc, char* argv[])
{
  printf("bug20170331 starts\n");
  assert(top.s.n == 0);
  assert(top.r.n == 0);
  sc_start();
  assert(top.s.n == ITERATIONS);
  assert(top.r.n == ITERATIONS);
  printf("bug20170331 done\n");
  return 0;
}

/* ex: set softtabstop=8 tabstop=8 shiftwidth=8 expandtab: */
