// parallel_threads_counting.cc: regression test for parallel execution
// RD, 04/21/17

#include <stdio.h>
#include <assert.h>

#include "systemc.h"

#define ITERATIONS 10
#define WORKLOAD   10000000

int Counter = 0; // global counter, alternatively incremented

unsigned long Work(int Load)
{
    int Count;
    unsigned long Result = 42;

    for(Count=0; Count<Load; Count++)
    {
	Result *= 17;
    }
    return Result;
}

SC_MODULE(Worker1) // counts to odd numbers at odd times
{
    unsigned Count;
    unsigned long w;

    void main(void)
    {
	wait(1, SC_NS);
	Count++;
	Counter++;
	assert(Counter == Count);
	w = Work(WORKLOAD);
	for(int i=0; i<ITERATIONS; i++)
	{
	    wait(2, SC_NS);
	    Count += 2;
	    Counter++;
	    assert(Counter == Count);
	    w = Work(WORKLOAD);
	}
	assert(Count == 1+2*ITERATIONS);
	assert(Count == Counter);
        sc_time expected_time(1+2*ITERATIONS, SC_NS);
	assert(sc_time_stamp() == expected_time);
    }

    SC_CTOR(Worker1)
    {
	Count = 0;
	SC_HAS_PROCESS(Worker1);
	SC_THREAD(main);
    }
};

SC_MODULE(Worker2) // counts to even numbers at even times
{
    unsigned Count;
    unsigned long w;

    void main(void)
    {
	wait(2, SC_NS);
	Count += 2;
	Counter++;
	assert(Counter == Count);
	w = Work(WORKLOAD);
	for(int i=0; i<ITERATIONS; i++)
	{
	    wait(2, SC_NS);
	    Count += 2;
	    Counter++;
	    assert(Counter == Count);
	    w = Work(WORKLOAD);
	}
	assert(Count == 2+2*ITERATIONS);
	assert(Count == Counter);
        sc_time expected_time(2+2*ITERATIONS, SC_NS);
	assert(sc_time_stamp() == expected_time);
    }

    SC_CTOR(Worker2)
    {
	Count = 0;
	SC_HAS_PROCESS(Worker2);
	SC_THREAD(main);
    }
};

SC_MODULE(TOP)
{
    Worker1 w1;
    Worker2 w2;

    SC_CTOR(TOP)
    : w1("w1"), w2("w2")
    {
    }
};

TOP top("top");

int sc_main(int argc, char* argv[])
{
    printf("parallel_threads_counting starts\n");
    sc_start();
    assert(top.w1.Count == 1+2*ITERATIONS);
    assert(top.w2.Count == 2+2*ITERATIONS);
    assert(Counter == 2+2*ITERATIONS);
    sc_time expected_time(2+2*ITERATIONS, SC_NS);
    assert(sc_time_stamp() == expected_time);
    printf("parallel_threads_counting done.\n");
    printf("[potential speedup over SEQ: OOO 1x, NPD 1x, SYN 1x]\n");
    return 0;
}

// EOF

/* ex: set softtabstop=8 tabstop=8 shiftwidth=8 expandtab: */
