#include <iostream>

#include "risc.h"

int main(int argc, char* argv[])
{
  std::cout << std::endl
            << "start processing file: " __FILE__
            << " ..."
            << std::endl;

  risc::Design d(argc,argv);

  // iterating all modules in the design
  for(risc::ModuleVectorIter
      iter_module  = d.module_definitions_.begin();
      iter_module != d.module_definitions_.end();
      iter_module++) {

    std::cout << "Module: " <<  (*iter_module)->get_name() << std::endl;

    for(risc::ThreadVectorIter
        iter_thread = (*iter_module)->threads_.begin();
        iter_thread != (*iter_module)->threads_.end();
        iter_thread++) {

      std::cout << (*iter_thread)->get_name() << std::endl;

      risc::sg::SegmentGraph::CCxxKeywordsBoundarySet keyword_boundary;

      keyword_boundary.insert(V_SgWhileStmt);
      keyword_boundary.insert(V_SgDoWhileStmt);
      keyword_boundary.insert(V_SgForStatement);
      keyword_boundary.insert(V_SgIfStmt);
      keyword_boundary.insert(V_SgSwitchStatement);

      risc::sg::SegmentGraph sg(keyword_boundary);

      sg.add_function(*iter_thread, *iter_thread);

      sg.clean_graph();
      sg.print_graph((*iter_thread)->get_name() + ".dot");

      //AstDOTGeneration dotgen;
      //dotgen.generate((*iter_thread)->get_ast_node(), (*iter_thread)->get_name());

      std::cout << std::endl;
    }

    std::cout << std::endl;
    risc::sg::Segment::id_counter = 0;
  }


  std::cout << "end processing file: " __FILE__
            << " ..."
            << std::endl
            << std::endl;

  return 0;
}

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
