#include "systemc.h"

class Sender: public sc_module {
public:

  sc_fifo_out<int> fifo_out_1;
  sc_fifo_out<int> fifo_out_2;

  SC_HAS_PROCESS(Sender);

  Sender(sc_module_name mn):
    sc_module(mn)
  {
    SC_THREAD(thread1);
    SC_THREAD(thread2);
  }

  void thread1()
  {
     fifo_out_1.write(17);
  }

  void thread2()
  {
    fifo_out_2->write(17);
  }
};

class Receiver: public sc_module {
public:

  sc_fifo_in<int> fifo_in_1;
  sc_fifo_in<int> fifo_in_2;

  SC_HAS_PROCESS(Receiver);

  Receiver(sc_module_name mn):
    sc_module(mn)
  {
    SC_THREAD(thread1);
    SC_THREAD(thread2);
  }

  void thread1()
  {
    int value = fifo_in_1.read();
    std::cout << "Read: " << value << std::endl;
  }

  void thread2()
  {
    int value = fifo_in_2->read();
    std::cout << "Read: " << value << std::endl;
  }
};

class Top: public sc_module {
public:

  // This one tests the usage of the . and -> operator for fifo ports
  Sender sender;
  Receiver receiver;

  sc_fifo<int> fifo1;
  sc_fifo<int> fifo2;

  SC_HAS_PROCESS(Top);

  Top(sc_module_name mn):
    sender("sender"),
    receiver("receiver"),
    sc_module(mn)
  { }

  void before_end_of_elaboration()
  {
    sender.fifo_out_1.bind(fifo1);
    sender.fifo_out_2.bind(fifo2);

    receiver.fifo_in_1.bind(fifo1);
    receiver.fifo_in_2.bind(fifo2);
  }
};

int sc_main(int argc, char **argv)
{
  Top t("top");
  sc_start();
  return 0;
}
